#version 330 core

in vec2 UV;

layout (location = 0) out vec3 color0;
layout (location = 1) out vec3 color1;

uniform sampler2D myTextureSampler;

uniform float threshold;

float getLuminanceFromRGB(vec3 rgb);
float brightPassValue(float luminance, out float alpha);

void main(){
	color0 = texture2D( myTextureSampler, UV ).rgb;
	
	float luminance = getLuminanceFromRGB(color0);
	float alpha = 0.0;
	luminance = brightPassValue(luminance, alpha);
	
	color1 = color0 * luminance;
}

float brightPassValue(float luminance, out float alpha)
{
	if( luminance > threshold) {
		alpha = 1.0;
		return max(luminance * 4, 1.0);
	}
	else {
		alpha = 0.0;
		return max(luminance * 0.0, 0.0);
	}
}

float getLuminanceFromRGB(vec3 rgb)
{	
	float minRGB = min( rgb.r, min(rgb.g, rgb.b) );
	
	float maxRGB = max( rgb.r, max(rgb.g, rgb.b) );

	return ( maxRGB + minRGB ) / 2.0;
}